@@ -8,6 +8,8 @@ from typing import Union, List, Dict, Any, Optional, cast, Tuple
 import math 
 import numpy as np
 
+from benchay.codes.astrape_pkg.astrape.utilities.utils import rearrange_dims
+
 applicable_val_metric_list = APPLICABLE_VAL_METRIC_LIST
 applicable_optimizer_type_list = APPLICABLE_OPTIMIZER_TYPE_LIST
 
@@ -122,7 +124,8 @@ class BaseNet(pl.LightningModule):
             self.log('train/loss', loss)
             self.log('train/acc', acc)
             self.log('train/auc', auc)
-            return {'loss' : loss, 'acc' : acc, 'auc' : auc}
+            log_dict = {'train/loss' : loss, 'train/acc' : acc, 'train/auc' : auc}
+            return {'loss' : loss, 'acc' : acc, 'auc' : auc, "progress_bar" : log_dict}
     
     def validation_step(self, val_batch, batch_idx):
         x_batch, y_batch = val_batch
@@ -141,11 +144,14 @@ class BaseNet(pl.LightningModule):
         else:    
             loss, acc, auc = self.criterion(Yhat.squeeze().type_as(y_batch), y_batch)
 
-            self.log('val/loss', loss)
-            self.log('val/acc', acc)
-            self.log('val/auc', auc)
             return {'loss' : loss, 'acc' : acc, 'auc' : auc}
-    
+    def validation_epoch_end(self, outputs):
+        loss = torch.stack([x["loss"] for x in outputs]).mean()
+        acc = torch.stack([x["acc"] for x in outputs]).mean()
+        auc = torch.stack([x["auc"] for x in outputs]).mean()
+        log_dict = {'val/loss' : loss, 'val/acc' : acc, 'val/auc' : auc}
+        return {"log": log_dict, "progress_bar" : log_dict}
+
     def predict_step(self, pred_batch, batch_idx, dataloader_idx=0):
         x_batch, y_batch = pred_batch
         
@@ -251,16 +257,14 @@ class ConvNet(pl.LightningModule):
 """
     def __init__(
         self,
-        model_type : "pl.LightningModule",
         n_classes : Optional[int] = None,
         sample_weight : Optional[Union["np.ndarray", "torch.Tensor"]] = None,
+        dataformats : Optional[str] = "NCHW",
         **config : Any
     )->None:
         r"""
         Args: 
 
-            model_type (LightningModule): The class of the model that you will use.
-
             n_classes (int, optional): Number of classes in the classification task. Leave it as None in regression tasks.
                 Default: ``None``
             
@@ -271,9 +275,9 @@ class ConvNet(pl.LightningModule):
         """
         super(ConvNet, self).__init__()
         
-        self.model_type = model_type
         self.n_classes = n_classes
         self.sample_weight = sample_weight
+        self.dataformats = dataformats
         self.save_hyperparameters(*config)
         
 
@@ -298,29 +302,23 @@ class ConvNet(pl.LightningModule):
             else:
                 criterion = MSELoss()
         else:
-            if self.model_type == "UNet":
-                criterion = CEDiceLossMetrics(self.n_classes)
-            else:
-                criterion = CELossMetrics(self.n_classes) if self.n_classes > 2 else BCELossMetrics()
+            criterion = CELossMetrics(self.n_classes) if self.n_classes > 2 else BCELossMetrics()
         return criterion
 
-    @staticmethod
-    def reshape_x(x_batch):
+    def reshape_x(self, x_batch):
         dims = x_batch.shape[1:]
         
         if len(dims) == 1:
-            width = height = math.sqrt(dims[0])
-            if (height != int(height)):
-                raise ValueError("Wrong dimension for the input.")
-            x_batch = x_batch.reshape(-1, 1, int(height), int(width))
+            raise ValueError(f"Wrong dimension {dims} for the input.")
         elif len(dims) == 2:
             height = dims[0]
             width = dims[1]
             x_batch = x_batch.reshape(-1, 1, int(height), int(width))
         elif len(dims) == 3:
-            x_batch = x_batch.reshape(-1, int(dims[0]), int(dims[1]), int(dims[2]))
+            dims = rearrange_dims(dims=dims, dataformats=self.dataformats)
+            x_batch = x_batch.reshape(-1, *dims)                
         else:
-            raise ValueError("Wrong dimension for the input.")
+            raise ValueError(f"Wrong dimension {dims} for the input.")
 
         return x_batch
 
@@ -328,13 +326,16 @@ class ConvNet(pl.LightningModule):
         batch[0] = self.reshape_x(batch[0])
         return batch
     
-    
+    def on_after_batch_transfer(self, batch, dataloader_idx=0):
+        # modify this part if you want to perform model reweigthing.
+        self.sample_weight = torch.ones_like(batch[1]).type_as(batch[1])
+        return batch
     
     def training_step(self, train_batch, batch_idx):
         x_batch, y_batch = train_batch
         x_batch = self.reshape_x(x_batch)
             
-        
+        y_batch = self.sample_weight * y_batch
 
         Yhat = self.forward(x_batch)
         if self.n_classes is None:
@@ -349,7 +350,9 @@ class ConvNet(pl.LightningModule):
             self.log('train/loss', loss)
             self.log('train/acc', acc)
             self.log('train/auc', auc)
-            return {'loss' : loss, 'acc' : acc, 'auc' : auc}
+            log_dict = {'train/loss' : loss, 'train/acc' : acc, 'train/auc' : auc}
+            return {'loss' : loss, 'acc' : acc, 'auc' : auc, "progress_bar" : log_dict}
+
 
         
     def validation_step(self, val_batch, batch_idx):
@@ -366,11 +369,15 @@ class ConvNet(pl.LightningModule):
             else:
                 loss, acc, auc = self.criterion(Yhat.squeeze(), y_batch.type(torch.FloatTensor))
 
-            self.log('val/loss', loss)
-            self.log('val/acc', acc)
-            self.log('val/auc', auc)
             return {'loss' : loss, 'acc' : acc, 'auc' : auc}
 
+    def validation_epoch_end(self, outputs):
+        loss = torch.stack([x["loss"] for x in outputs]).mean()
+        acc = torch.stack([x["acc"] for x in outputs]).mean()
+        auc = torch.stack([x["auc"] for x in outputs]).mean()
+        log_dict = {'val/loss' : loss, 'val/acc' : acc, 'val/auc' : auc}
+        return {"log": log_dict, "progress_bar" : log_dict}
+
     def predict_step(self, pred_batch, batch_idx, dataloader_idx=0):
         x_batch, y_batch = pred_batch
         x_batch = self.reshape_x(x_batch)
